__version__ = "5.2.0"

from .smartcar import (
    AuthClient,
    is_expired,
    get_user_id,
    get_vehicle_ids,
    set_api_version,
)
from .vehicle import Vehicle
from .exceptions import (
    SmartcarException,
    SmartcarExceptionV2,
    ValidationException,
    AuthenticationException,
    PermissionException,
    ResourceNotFoundException,
    StateException,
    RateLimitingException,
    MonthlyLimitExceeded,
    ServerException,
    VehicleNotCapableException,
    SmartcarNotCapableException,
    GatewayTimeoutException,
)
